@@ -9,8 +9,11 @@
 #' @param dir The directory to install or uninstall TinyTeX (should not exist
 #'   unless \code{force = TRUE}).
 #' @param version The version of TinyTeX, e.g., \code{"2020.09"} (see all
-#'   available versions at \url{https://github.com/yihui/tinytex-releases}). By
-#'   default, it installs the latest daily build of TinyTeX.
+#'   available versions at \url{https://github.com/yihui/tinytex-releases}, or
+#'   the last few releases via
+#'   \code{xfun::github_releases('yihui/tinytex-releases')}). By default, it
+#'   installs the latest daily build of TinyTeX. If \code{version = 'latest'},
+#'   it installs the latest Github release of TinyTeX.
 #' @param repository The CTAN repository to set. You can find available
 #'   repositories at \code{https://ctan.org/mirrors}), e.g.,
 #'   \code{'http://mirrors.tuna.tsinghua.edu.cn/CTAN/'}, or
@@ -27,7 +30,7 @@
 #'   for the default installation directories on different platforms.
 #' @export
 install_tinytex = function(
-  force = FALSE, dir = 'auto', version = '', repository = 'ctan',
+  force = FALSE, dir = 'auto', version = 'daily', repository = 'ctan',
   extra_packages = if (is_tinytex()) tl_pkgs(), add_path = TRUE
 ) {
   if (!is.logical(force)) stop('The argument "force" must take a logical value.')
@@ -72,13 +75,14 @@ install_tinytex = function(
   )
 
   install = function(...) {
-    if (need_source_install()) {
+    if (getOption('tinytex.source.install', need_source_install())) {
       install_tinytex_source(repository, ...)
     } else {
-      install_prebuilt('TinyTeX-1', ...)
+      install_prebuilt('TinyTeX-1', ..., repo = repository)
     }
   }
   force(extra_packages)  # evaluate it before installing another version of TinyTeX
+  if (version == 'daily') version = ''
   user_dir = install(user_dir, version, add_path, extra_packages)
 
   opts = options(tinytex.tlmgr.path = find_tlmgr(user_dir))
@@ -162,7 +166,7 @@ install_tinytex_source = function(repo = '', dir, version, add_path, extra_packa
   }
   opts = options(tinytex.tlmgr.path = find_tlmgr(target))
   on.exit(options(opts), add = TRUE)
-  post_install_config(add_path, extra_packages)
+  post_install_config(add_path, extra_packages, repo)
   unlink(c('install-unx.sh', 'install-tl.zip', 'pkgs-custom.txt', 'tinytex.profile'))
   target
 }
@@ -270,6 +274,11 @@ symlink_root = function(path) {
   in_dir(dirname(path), symlink_root(path2))
 }
 
+# a helper function to open tlmgr.pl (on *nix)
+open_tlmgr = function() {
+  file.edit(symlink_root(Sys.which('tlmgr')))
+}
+
 #' Check if the LaTeX installation is TinyTeX
 #'
 #' First find the root directory of the installation via
@@ -313,7 +322,8 @@ install_yihui_pkgs = function() {
 
 # install a prebuilt version of TinyTeX
 install_prebuilt = function(
-  pkg = '', dir = '', version = '', add_path = TRUE, extra_packages = NULL, hash = FALSE, cache = NA
+  pkg = '', dir = '', version = '', add_path = TRUE, extra_packages = NULL,
+  repo = 'ctan', hash = FALSE, cache = NA
 ) {
   if (need_source_install()) stop(
     'There is no prebuilt version of TinyTeX for this platform: ',
@@ -327,6 +337,10 @@ install_prebuilt = function(
   dir2 = file.path(target, b)  # path to (.)TinyTeX/ after extraction
 
   if (xfun::file_ext(pkg) == '') {
+    if (version == 'latest') {
+      version = xfun::github_releases('yihui/tinytex-releases', version)
+      version = gsub('^v', '', version)
+    }
     installer = if (pkg == '') 'TinyTeX' else pkg
     # e.g., TinyTeX-0.zip, TinyTeX-1-v2020.10.tar.gz, ...
     pkg = paste0(
@@ -338,7 +352,9 @@ install_prebuilt = function(
       if (as.numeric(difftime(Sys.time(), file.mtime(pkg), units = 'days')) > 1)
         cache = FALSE
     }
-    if (xfun::isFALSE(cache)) file.remove(pkg)
+    if (xfun::isFALSE(cache)) {
+      file.remove(pkg); on.exit(file.remove(pkg), add = TRUE)
+    }
     if (!file.exists(pkg)) download_installer(pkg, version)
   }
 
@@ -352,18 +368,21 @@ install_prebuilt = function(
     opts = options(tinytex.tlmgr.path = find_tlmgr(dir1))
     on.exit(options(opts), add = TRUE)
   }
-  post_install_config(add_path, extra_packages, hash)
+  post_install_config(add_path, extra_packages, repo, hash)
   invisible(dir1)
 }
 
 # post-install configurations
-post_install_config = function(add_path, extra_packages, hash = FALSE) {
+post_install_config = function(add_path, extra_packages, repo, hash = FALSE) {
   if (os_index == 2) {
     dir.create('~/bin', FALSE, TRUE)
     tlmgr(c('option', 'sys_bin', '~/bin'))
   }
   if (add_path) tlmgr_path()
   r_texmf(.quiet = TRUE)
+  # don't use the default random ctan mirror when installing on CI servers
+  if (repo != 'ctan' || tolower(Sys.getenv('CI')) != 'true')
+    tlmgr_repo(repo, stdout = FALSE, .quiet = TRUE)
   tlmgr_install(setdiff(extra_packages, tl_pkgs()))
   if (hash) {
     texhash(); fmtutil(stdout = FALSE); updmap(); fc_cache()
